---
title: "(Optional) Ruler"
description: "Overview of the ruler microservice."
weight: 20
---

# (Optional) Ruler

The **ruler** is an **optional service** that executes PromQL queries for recording rules and alerts. The ruler requires object storage to store the recording rules and alerts for each tenant.

Ruler is **semi-stateful** and can be scaled horizontally.
Running rules internally have state, as well as the ring the rulers initiate.
However, if the rulers all fail and restart,
Prometheus alert rules have a feature where an alert is restored and returned to a firing state
if it would have been active in its for period.
However, there would be gaps in the series generated by the recording rules.

One option to scale the ruler is by scaling it horizontally. However, with multiple ruler instances running they will need to coordinate to determine which instance will evaluate which rule. Similar to the ingesters, the rulers establish a hash ring to divide up the responsibilities of evaluating rules.

## Storage

The ruler supports several kinds of storage (azure, gcs, s3, swift, filesystem, local). Most kinds of storage work with the sharded ruler configuration in an obvious way. i.e. configure all rulers to use the same backend.

> **Note:** The `filesystem` storage backend should only be used for testing purposes.

The local implementation reads [Prometheus recording rules](https://prometheus.io/docs/prometheus/latest/configuration/recording_rules/) off of the local filesystem. This is a read only backend that does not support the creation and deletion of rules through [the API](../api/_index.md#ruler). Despite the fact that it reads the local filesystem this method can still be used in a sharded ruler configuration if the operator takes care to load the same rules to every ruler. For instance this could be accomplished by mounting a [Kubernetes ConfigMap](https://kubernetes.io/docs/concepts/configuration/configmap/) onto every ruler pod.

A typical local config may look something like:

```
  -ruler-storage.backend=local
  -ruler-storage.local.directory=/tmp/cortex/rules
```

With the above configuration the ruler would expect the following layout:

```
/tmp/cortex/rules/<tenant id>/rules1.yaml
                             /rules2.yaml
```

Yaml files are expected to be in the [Prometheus format](https://prometheus.io/docs/prometheus/latest/configuration/recording_rules/#recording-rules).
